package org.eclipse.jdt.internal.compiler.apt.dispatch;

import java.util.regex.Matcher;
import javax.lang.model.element.TypeElement;
import java.util.Iterator;
import java.util.regex.Pattern;
import javax.lang.model.SourceVersion;
import java.util.Set;
import javax.annotation.processing.Processor;

public class ProcessorInfo
{
    final Processor _processor;
    final Set<String> _supportedOptions;
    final SourceVersion _supportedSourceVersion;
    private final Pattern _supportedAnnotationTypesPattern;
    private final boolean _supportsStar;
    private boolean _hasBeenCalled;
    
    public ProcessorInfo(Processor p) {
        this._processor = p;
        this._hasBeenCalled = false;
        this._supportedSourceVersion = p.getSupportedSourceVersion();
        this._supportedOptions = p.getSupportedOptions();
        Set<String> supportedAnnotationTypes = p.getSupportedAnnotationTypes();
        boolean supportsStar = false;
        if (supportedAnnotationTypes != null && !supportedAnnotationTypes.isEmpty()) {
            StringBuilder regex = new StringBuilder();
            Iterator<String> iName = supportedAnnotationTypes.iterator();
            while (true) {
                String name = (String)iName.next();
                supportsStar |= "*".equals(name);
                String escapedName1 = name.replace(".", "\\.");
                String escapedName2 = escapedName1.replace("*", ".*");
                regex.append(escapedName2);
                if (!iName.hasNext()) {
                    break;
                }
                regex.append('|');
            }
            this._supportedAnnotationTypesPattern = Pattern.compile(regex.toString());
        }
        else {
            this._supportedAnnotationTypesPattern = null;
        }
        this._supportsStar = supportsStar;
    }
    
    public boolean computeSupportedAnnotations(Set<TypeElement> annotations, Set<TypeElement> result) {
        if (annotations != null && !annotations.isEmpty() && this._supportedAnnotationTypesPattern != null) {
            for (TypeElement annotation : annotations) {
                Matcher matcher = this._supportedAnnotationTypesPattern.matcher(annotation.getQualifiedName().toString());
                if (matcher.matches()) {
                    result.add(annotation);
                }
                else {
                    continue;
                }
            }
        }
        boolean call = this._hasBeenCalled || this._supportsStar || !result.isEmpty();
        this._hasBeenCalled |= call;
        return call;
    }
    
    public boolean supportsStar() {
        return this._supportsStar;
    }
    
    public void reset() {
        this._hasBeenCalled = false;
    }
    
    @Override
    public int hashCode() {
        return this._processor.getClass().hashCode();
    }
    
    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (this.getClass() != obj.getClass()) {
            return false;
        }
        ProcessorInfo other = (ProcessorInfo)obj;
        return this._processor.getClass().equals(other._processor.getClass());
    }
    
    @Override
    public String toString() {
        return this._processor.getClass().getName();
    }
    
    public String getSupportedAnnotationTypesAsString() {
        StringBuilder sb = new StringBuilder();
        sb.append('[');
        Iterator<String> iAnnots = this._processor.getSupportedAnnotationTypes().iterator();
        boolean hasNext = iAnnots.hasNext();
        while (hasNext) {
            sb.append((String)iAnnots.next());
            hasNext = iAnnots.hasNext();
            if (hasNext) {
                sb.append(',');
            }
            else {
                continue;
            }
        }
        sb.append(']');
        return sb.toString();
    }
}
